! ###########################################################
!                 SDSU v2 LUT creation routines
!                       - mktab_mwr -
!
! This subroutine creates the Mie lookup table for the
! microwave radiometer simulator.
!
!                              Created by H. M. (May 2009)
!
! Since SDSU v2.1.0, loops over "npts_ncnc" has been updated
! so a two moment scheme can be readily implemented.
!                                         H. M. (Aug 2009)
!
 Subroutine mktab_mwr
   Use LUTparameters
   Use MCRPparameters
   Implicit NONE
!
! Local variables
!
   real :: frequency
   real :: kext,salb,asym,pbck
   integer :: nspec,i,j,nfreq,k
   integer :: npts_ncnc0,npts_ncnc1
   character*50  :: fname_out(0:6),fname_lab(0:6)
   character*100 :: pathandfile
   character*4, external :: mklab_lut
   character*4, dimension(:), allocatable :: fnametag_rdmtr
   real, dimension(:,:,:), allocatable :: kext_lut,salb_lut,asym_lut
!
   write(*,*) '- Microwave Radiometer- '
!
!  Frequency name tags for the output lookup tables
!  defined by "src/simulator/hydromet/mklab_lut.f90".
!
   allocate ( fnametag_rdmtr(nfreqMW) )
   do nfreq = 1, nfreqMW
      fnametag_rdmtr(nfreq) = mklab_lut ( freq_rdmtr(nfreq) )
   end do
!
! Info files
!
   fname_lab(0) ='melt-p/'//snow%functype//trim(snow%name)//'_' & 
                          //rain%functype//trim(rain%name)//'-mwr'
   fname_lab(1) = cldi%functype//'/'//trim(cldi%name)//'-mwr'
   fname_lab(2) = cldw%functype//'/'//trim(cldw%name)//'-mwr'
   fname_lab(3) = grpl%functype//'/'//trim(grpl%name)//'-mwr'
   fname_lab(4) = hail%functype//'/'//trim(hail%name)//'-mwr'
   fname_lab(5) = snow%functype//'/'//trim(snow%name)//'-mwr'
   fname_lab(6) = rain%functype//'/'//trim(rain%name)//'-mwr'
!
   pathandfile = dir_out//trim(fname_lab(1))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Cloud ice: ',trim(cldi%name)//'('//cldi%functype//')'
   Do i = 1, cldi%nparams
      write(15,10) '  PSD parameter #',i,'=',cldi%paramset(i)
   End do
!
   pathandfile = dir_out//trim(fname_lab(2))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Cloud water: ',trim(cldw%name)//'('//cldw%functype//')'
   Do i = 1, cldw%nparams
      write(15,10) '  PSD parameter #',i,'=',cldw%paramset(i)
   End do
!
   pathandfile = dir_out//trim(fname_lab(3))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Graupel: ',trim(grpl%name)//'('//grpl%functype//')'
   Do i = 1, grpl%nparams
      write(15,10) '  PSD parameter #',i,'=',grpl%paramset(i)
   End do
!
   pathandfile = dir_out//trim(fname_lab(4))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Hail: ',trim(hail%name)//'('//hail%functype//')'
   Do i = 1, hail%nparams
      write(15,10) '  PSD parameter #',i,'=',hail%paramset(i)
   End do
!
   pathandfile = dir_out//trim(fname_lab(5))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Snow: ',trim(snow%name)//'('//snow%functype//')'
   Do i = 1, snow%nparams
      write(15,10) '  PSD parameter #',i,'=',snow%paramset(i)
   End do
!
   pathandfile = dir_out//trim(fname_lab(6))//'.info'
   open ( unit=15, file=pathandfile )
   write(15,*) ' - Rain: ',trim(rain%name)//'('//rain%functype//')'
   Do i = 1, rain%nparams
      write(15,10) '  PSD parameter #',i,'=',rain%paramset(i)
   End do
!
10 format(a,i2,a,1P1e10.3)
!
! Create LUTs
!
   do nfreq = 1 , nfreqMW
!
      write(*,*) 'Freq = ',freq_rdmtr(nfreq)
!
      do nspec = 0, 6
         fname_out(nspec) = trim(fname_lab(nspec))// &
                            fnametag_rdmtr(nfreq)//'.dat' 
      end do
!
      frequency = freq_rdmtr(nfreq)
!
! Melting particles
!
      allocate ( kext_lut(npts_hydro,npts_ncnc,npts_ncnc) )
      allocate ( salb_lut(npts_hydro,npts_ncnc,npts_ncnc) )
      allocate ( asym_lut(npts_hydro,npts_ncnc,npts_ncnc) )
!
      nspec = 0
!
      if ( rain%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
      if ( snow%twomomnt ) then
         npts_ncnc1 = npts_ncnc
      else
         npts_ncnc1 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Melting particles -'
! 
!       Write header for output file
!
      write(2,*) npts_hydro, npts_ncnc0, npts_ncnc1
      write(2,92) (pts_hydro(j),j=1,npts_hydro)
      if ( rain%twomomnt ) then
         write(2,92) (pts_ncncp(k),k=1,npts_ncnc)
      else
         write(2,92)  pts_ncncp(1)
      endif
      if ( snow%twomomnt ) then
         write(2,92) (pts_ncncp(k),k=1,npts_ncnc)
      else
         write(2,92)  pts_ncncp(1)
      endif
      write(2,91) frequency
!
      do k = 1, npts_ncnc1
         do i = 1, npts_ncnc0
            do j = 1, npts_hydro
               Call df_mie_melt(frequency,.5*pts_hydro(j),.5*pts_hydro(j), &
                                             pts_ncncp(i),   pts_ncncp(k), &
                                 kext,asym,salb,pbck)
               kext_lut(j,i,k) = kext
               salb_lut(j,i,k) = salb
               asym_lut(j,i,k) = asym
            end do
         end do
      end do
      write(2,90) (((kext_lut(j,i,k),salb_lut(j,i,k),asym_lut(j,i,k), &
                     j=1,npts_hydro),i=1,npts_ncnc0),k=1,npts_ncnc1)
      close(2)
!
      deallocate ( kext_lut, salb_lut, asym_lut )
      allocate ( kext_lut(npts_hydro,npts_temp,npts_ncnc) )
      allocate ( salb_lut(npts_hydro,npts_temp,npts_ncnc) )
      allocate ( asym_lut(npts_hydro,npts_temp,npts_ncnc) )
!
! cloud ice
!
      nspec = 1
!
      if ( cldi%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Cloud ice -'
!
!       Write header for output file
!
      write(2,95) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,92) (pts_ncncc(k),k=1,npts_ncnc0)
      write(2,91) frequency
!
!       Calculate scattering parameters
!
      do k = 1, npts_ncnc0
         do i = 1, npts_temp
            do j  = 1, npts_hydro
!
               call mie_cldi(frequency, &
                             pts_temp_i(i),pts_hydro(j),pts_ncncc(k), &
                             kext,salb,asym,pbck)
               kext_lut(j,i,k) = kext
               salb_lut(j,i,k) = salb
               asym_lut(j,i,k) = asym
!
            end do
         end do
      end do
      write(2,90) (((kext_lut(j,i,k),salb_lut(j,i,k),asym_lut(j,i,k), &
                     j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! cloud water
!
      nspec = 2
!
      if ( cldw%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Cloud water -'
!
!       Write header for output file
!
      write(2,95) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_w(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,92) (pts_ncncc(k),k=1,npts_ncnc0)
      write(2,91) frequency
!
!       Calculate scattering parameters
!
      do k = 1, npts_ncnc0
         do i = 1, npts_temp
            do j  = 1, npts_hydro
!
               call mie_cldw(frequency, &
                             pts_temp_w(i),pts_hydro(j),pts_ncncc(k), &
                             kext,salb,asym,pbck)
               kext_lut(j,i,k) = kext
               salb_lut(j,i,k) = salb
               asym_lut(j,i,k) = asym
!
            end do
         end do
      end do
      write(2,90) (((kext_lut(j,i,k),salb_lut(j,i,k),asym_lut(j,i,k), &
                     j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! graupel
!
      nspec = 3
!
      if ( grpl%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Graupel -'
!
!       Write header for output file
!
      write(2,95) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,92) (pts_ncncp(k),k=1,npts_ncnc0)
      write(2,91) frequency
!
!       Calculate scattering parameters
!
      do k = 1, npts_ncnc0
         do i = 1, npts_temp
            do j  = 1, npts_hydro
!
               call mie_grpl(frequency, &
                             pts_temp_i(i),pts_hydro(j),pts_ncncp(k), &
                             kext,salb,asym,pbck)
               kext_lut(j,i,k) = kext
               salb_lut(j,i,k) = salb
               asym_lut(j,i,k) = asym
!
            end do
         end do
      end do
      write(2,90) (((kext_lut(j,i,k),salb_lut(j,i,k),asym_lut(j,i,k), &
                     j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! hail
!
      nspec = 4
!
      if ( hail%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Hail -'
!
!       Write header for output file
!
      write(2,95) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,92) (pts_ncncp(k),k=1,npts_ncnc0)
      write(2,91) frequency
!
!       Calculate scattering parameters
!
      do k = 1, npts_ncnc0
         do i = 1, npts_temp
            do j  = 1, npts_hydro
!
               call mie_hail(frequency, &
                             pts_temp_i(i),pts_hydro(j),pts_ncncp(k), &
                             kext,salb,asym,pbck)
               kext_lut(j,i,k) = kext
               salb_lut(j,i,k) = salb
               asym_lut(j,i,k) = asym
!
            end do
         end do
      end do
      write(2,90) (((kext_lut(j,i,k),salb_lut(j,i,k),asym_lut(j,i,k), &
                     j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! snow
!
      nspec = 5
!
      if ( snow%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Snow -'
!
!       Write header for output file
!
      write(2,95) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,92) (pts_ncncp(k),k=1,npts_ncnc0)
      write(2,91) frequency
!
!       Calculate scattering parameters
!
      do k = 1, npts_ncnc0
         do i = 1, npts_temp
            do j  = 1, npts_hydro
!
               call mie_snow(frequency, &
                             pts_temp_i(i),pts_hydro(j),pts_ncncp(k), &
                             kext,salb,asym,pbck)
               kext_lut(j,i,k) = kext
               salb_lut(j,i,k) = salb
               asym_lut(j,i,k) = asym
!
            end do
         end do
      end do
      write(2,90) (((kext_lut(j,i,k),salb_lut(j,i,k),asym_lut(j,i,k), &
                     j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! Rain water
!
      nspec = 6
!
      if ( rain%twomomnt ) then
         npts_ncnc0 = npts_ncnc
      else
         npts_ncnc0 = 1
      endif
!
      pathandfile = dir_out//trim(fname_out(nspec))
      open(2,file=pathandfile)
      write(*,*) 'Rain -'
!
!       Write header for output file
!
      write(2,95) npts_temp, npts_hydro, npts_ncnc0
      write(2,92) (pts_temp_w(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,92) (pts_ncncp(k),k=1,npts_ncnc0)
      write(2,91) frequency
!
!       Calculate scattering parameters
!
      do k = 1, npts_ncnc0
         do i = 1, npts_temp
            do j  = 1, npts_hydro
!
               call mie_rain(frequency, &
                             pts_temp_w(i),pts_hydro(j),pts_ncncp(k), &
                             kext,salb,asym,pbck)
               kext_lut(j,i,k) = kext
               salb_lut(j,i,k) = salb
               asym_lut(j,i,k) = asym
!
            end do
         end do
      end do
      write(2,90) (((kext_lut(j,i,k),salb_lut(j,i,k),asym_lut(j,i,k), &
                     j=1,npts_hydro),i=1,npts_temp),k=1,npts_ncnc0)
      close (2)
!
! Output formatting
!
90 Format ( 1P3E14.6 )   ! kext, salb, and asym/pbck
91 Format ( 1F12.5 )     ! frequency
92 Format ( 1P50E13.5 )  ! LUT vectors
95 Format ( 3I5 )        ! LUT dimension sizes
!
      deallocate ( kext_lut, salb_lut, asym_lut )
!
   end do
!
   Return
 end Subroutine mktab_mwr
