! ###########################################################
!            Satellite Data Simulation Unit v2
!                     - MWRparameters -
!
! This module defines variables used for MWR simulator.
!
 Module MWRparameters
   Implicit NONE
!
! kext: extinction coefficient
! sabl: single scattering albedo
! asym: asymmetry parameter
!
   real, dimension(:,:,:), allocatable :: kexttot,salbtot,asymtot
!
! tb: brightness temperature 
! emis: surface emissivity
! ebar: angularly-averaged emissivity
!
   real, dimension(:,:), allocatable :: tb, emis, ebar
!
! kext, salb, and asym stored in the LUT for rain (rr), cloud ice (ci),
! cloud water (cw), snow (sn), graupel (gr), hail (hl), and melting
! particles (ml).
!
   real, dimension(:,:,:), allocatable :: kextrr_lut,salbrr_lut,asymrr_lut
   real, dimension(:,:,:), allocatable :: kextci_lut,salbci_lut,asymci_lut
   real, dimension(:,:,:), allocatable :: kextcw_lut,salbcw_lut,asymcw_lut
   real, dimension(:,:,:), allocatable :: kextsn_lut,salbsn_lut,asymsn_lut
   real, dimension(:,:,:), allocatable :: kextgr_lut,salbgr_lut,asymgr_lut
   real, dimension(:,:,:), allocatable :: kexthl_lut,salbhl_lut,asymhl_lut
   real, dimension(:,:,:), allocatable :: kextml_lut,salbml_lut,asymml_lut
!
! / Updated for SDSU v.2.1.0 /
! pts_hydro  : LUT hydrometeor water content vector (in [g/m3])
! ptn_????   : LUT hydrometeor number concentration vector (in [/m3])
!              ????=cldi, cldw, rain, snow, grpl, or hail
! pts_temp_w : LUT temperature vector (in [K]) for liquid hydrometeors
! pts_temp_i : LUT temperature vector (in [K]) for frozen hydrometeors
! npts_hydro : LUT hydrometeor water content dimension size
! npts_temp  : LUT temperature dimension size
! npts_ncnc  : LUT hydrometeor number concentration dimension size
! ( These parameters will be read in from the LUTs. )
!
   real, dimension(:), allocatable :: pts_hydro, pts_temp_w, pts_temp_i
   real, dimension(:), allocatable :: ptn_cldi, ptn_cldw, ptn_rain
   real, dimension(:), allocatable :: ptn_snow, ptn_grpl, ptn_hail
   integer :: npts_hydro, npts_temp, npts_ncnc
!
! Miscellaneous
!
   real :: umu
   logical :: lambert
!
 End Module MWRparameters
 
